@@ -1,7 +1,10 @@
+# Copyright (c) 2000-2002 Graham Barr <gbarr@pobox.com>. All rights reserved.
+# This program is free software; you can redistribute it and/or
+# modify it under the same terms as Perl itself.
 
 package Convert::ASN1;
 
-# $Id: _encode.pm,v 1.10 2001/08/01 18:02:38 gbarr Exp $
+# $Id: _encode.pm,v 1.13 2002/01/22 11:24:28 gbarr Exp $
 
 BEGIN {
   local $SIG{__DIE__};
@@ -28,51 +31,53 @@ my @encode = (
   \&_enc_time,
   \&_enc_utf8,
   \&_enc_any,
-  \&_enc_choice
+  \&_enc_choice,
+  \&_enc_object_id,
 );
 
 
 sub _encode {
-  my $optn  = shift;
-  my $ops   = shift;
-  my $stash = shift;
+  my ($optn, $ops, $stash, $path) = @_;
+  my $var;
 
   foreach my $op (@{$ops}) {
     if (defined(my $opt = $op->[cOPT])) {
       next unless defined $stash->{$opt};
     }
-    foreach my $var (defined($op->[cVAR]) ? $stash->{$op->[cVAR]} : undef) {
-      $_[0] .= $op->[cTAG];
-
-      die $op->[cVAR] unless defined($var) || !defined($op->[cVAR]);
-
-      &{$encode[$op->[cTYPE]]}(
-	$optn,
-	$op,
-	$stash,
-	$var,
-	$_[0],
-	$op->[cLOOP]
-      );
-
+    if (defined($var = $op->[cVAR])) {
+      push @$path, $var;
+      require Carp, Carp::croak(join(".", @$path)," is undefined")  unless defined $stash->{$var};
     }
+    $_[4] .= $op->[cTAG];
+
+    &{$encode[$op->[cTYPE]]}(
+      $optn,
+      $op,
+      $stash,
+      defined($var) ? $stash->{$var} : undef,
+      $_[4],
+      $op->[cLOOP],
+      $path,
+    );
+
+    pop @$path if defined $var;
   }
 
-  $_[0];
+  $_[4];
 }
 
 
 sub _enc_boolean {
-# 0      1    2       3     4     5
-# $optn, $op, $stash, $var, $buf, $loop
+# 0      1    2       3     4     5      6
+# $optn, $op, $stash, $var, $buf, $loop, $path
 
   $_[4] .= pack("CC",1, $_[3] ? 0xff : 0);
 }
 
 
 sub _enc_integer {
-# 0      1    2       3     4     5
-# $optn, $op, $stash, $var, $buf, $loop
+# 0      1    2       3     4     5      6
+# $optn, $op, $stash, $var, $buf, $loop, $path
   if (abs($_[3]) >= 2**31) {
     my $os = i2osp($_[3], ref($_[3]) || $_[0]->{encode_bigint} || 'Math::BigInt');
     my $len = length $os;
@@ -96,8 +101,8 @@ sub _enc_integer {
 
 
 sub _enc_bitstring {
-# 0      1    2       3     4     5
-# $optn, $op, $stash, $var, $buf, $loop
+# 0      1    2       3     4     5      6
+# $optn, $op, $stash, $var, $buf, $loop, $path
 
   if (ref($_[3])) {
     my $less = (8 - ($_[3]->[1] & 7)) & 7;
@@ -118,8 +123,8 @@ sub _enc_bitstring {
 
 
 sub _enc_string {
-# 0      1    2       3     4     5
-# $optn, $op, $stash, $var, $buf, $loop
+# 0      1    2       3     4     5      6
+# $optn, $op, $stash, $var, $buf, $loop, $path
 
   $_[4] .= asn_encode_length(length $_[3]);
   $_[4] .= $_[3];
@@ -127,25 +132,27 @@ sub _enc_string {
 
 
 sub _enc_null {
-# 0      1    2       3     4     5
-# $optn, $op, $stash, $var, $buf, $loop
+# 0      1    2       3     4     5      6
+# $optn, $op, $stash, $var, $buf, $loop, $path
 
   $_[4] .= chr(0);
 }
 
 
 sub _enc_object_id {
-# 0      1    2       3     4     5
-# $optn, $op, $stash, $var, $buf, $loop
+# 0      1    2       3     4     5      6
+# $optn, $op, $stash, $var, $buf, $loop, $path
 
   my @data = ($_[3] =~ /(\d+)/g);
 
-  if(@data < 2) {
+  if ($_[1]->[cTYPE] == opOBJID) {
+    if(@data < 2) {
       @data = (0);
-  }
-  else {
+    }
+    else {
       my $first = $data[1] + ($data[0] * 40);
       splice(@data,0,2,$first);
+    }
   }
 
   my $l = length $_[4];
@@ -155,8 +162,8 @@ sub _enc_object_id {
 
 
 sub _enc_real {
-# 0      1    2       3     4     5
-# $optn, $op, $stash, $var, $buf, $loop
+# 0      1    2       3     4     5      6
+# $optn, $op, $stash, $var, $buf, $loop, $path
 
   # Zero
   unless ($_[3]) {
@@ -222,8 +229,8 @@ sub _enc_real {
 
 
 sub _enc_sequence {
-# 0      1    2       3     4     5
-# $optn, $op, $stash, $var, $buf, $loop
+# 0      1    2       3     4     5      6
+# $optn, $op, $stash, $var, $buf, $loop, $path
 
   if (my $ops = $_[1]->[cCHILD]) {
     my $l = length $_[4];
@@ -234,7 +241,10 @@ sub _enc_sequence {
       my $tag  = $op->[cTAG];
       my $loop = $op->[cLOOP];
 
+      push @{$_[6]}, -1;
+
       foreach my $var (@{$_[3]}) {
+	$_[6]->[-1]++;
 	$_[4] .= $tag;
 
 	&{$enc}(
@@ -243,12 +253,14 @@ sub _enc_sequence {
 	  $_[2], # $stash
 	  $var,  # $var
 	  $_[4], # $buf
-	  $loop  # $loop
+	  $loop, # $loop
+	  $_[6], # $path
 	);
       }
+      pop @{$_[6]};
     }
     else {
-      _encode($_[0],$_[1]->[cCHILD], defined($_[3]) ? $_[3] : $_[2] , $_[4]);
+      _encode($_[0],$_[1]->[cCHILD], defined($_[3]) ? $_[3] : $_[2], $_[6], $_[4]);
     }
     substr($_[4],$l,2) = asn_encode_length(length($_[4]) - $l - 2);
   }
@@ -262,8 +274,8 @@ sub _enc_sequence {
 my %_enc_time_opt = ( utctime => 1, withzone => 0, raw => 2);
 
 sub _enc_time {
-# 0      1    2       3     4     5
-# $optn, $op, $stash, $var, $buf, $loop
+# 0      1    2       3     4     5      6
+# $optn, $op, $stash, $var, $buf, $loop, $path
 
   my $mode = $_enc_time_opt{$_[0]->{'encode_time'} || ''} || 0;
 
@@ -313,8 +325,8 @@ sub _enc_time {
 
 
 sub _enc_utf8 {
-# 0      1    2       3     4     5
-# $optn, $op, $stash, $var, $buf, $loop
+# 0      1    2       3     4     5      6
+# $optn, $op, $stash, $var, $buf, $loop, $path
 
   $_[4] .= asn_encode_length(length $_[3]);
   $_[4] .= $_[3];
@@ -322,26 +334,29 @@ sub _enc_utf8 {
 
 
 sub _enc_any {
-# 0      1    2       3     4     5
-# $optn, $op, $stash, $var, $buf, $loop
+# 0      1    2       3     4     5      6
+# $optn, $op, $stash, $var, $buf, $loop, $path
 
   $_[4] .= $_[3];
 }
 
 
 sub _enc_choice {
-# 0      1    2       3     4     5
-# $optn, $op, $stash, $var, $buf, $loop
+# 0      1    2       3     4     5      6
+# $optn, $op, $stash, $var, $buf, $loop, $path
 
   my $stash = defined($_[3]) ? $_[3] : $_[2];
   for my $op (@{$_[1]->[cCHILD]}) {
     my $var = $op->[cVAR];
     if (exists $stash->{$var}) {
-      _encode($_[0],[$op], $stash, $_[4]);
+      push @{$_[6]}, $var;
+      _encode($_[0],[$op], $stash, $_[6], $_[4]);
+      pop @{$_[6]};
       return;
     }
   }
-  die "No value found for CHOICE\n";
+  require Carp;
+  Carp::croak("No value found for CHOICE " . join(".", @{$_[6]}));
 }
 
 
