@@ -3,9 +3,13 @@
 
 # 20 "parser.y"
 
+;# Copyright (c) 2000-2002 Graham Barr <gbarr@pobox.com>. All rights reserved.
+;# This program is free software; you can redistribute it and/or
+;# modify it under the same terms as Perl itself.
+
 package Convert::ASN1::parser;
 
-;# $Id: parser.pm,v 1.5 2001/08/24 14:13:52 gbarr Exp $
+;# $Id: parser.pm,v 1.7 2002/01/22 11:24:28 gbarr Exp $
 
 use strict;
 use Convert::ASN1 qw(:all);
@@ -20,16 +24,17 @@ my $yydebug=0;
 my %yystate;
 
 my %base_type = (
-  BOOLEAN	    => [ asn_encode_tag(ASN_BOOLEAN),    opBOOLEAN ],
-  INTEGER	    => [ asn_encode_tag(ASN_INTEGER),    opINTEGER ],
-  BIT_STRING	    => [ asn_encode_tag(ASN_BIT_STR),    opBITSTR  ],
-  OCTET_STRING	    => [ asn_encode_tag(ASN_OCTET_STR),  opSTRING  ],
-  STRING	    => [ asn_encode_tag(ASN_OCTET_STR),  opSTRING  ],
-  NULL 		    => [ asn_encode_tag(ASN_NULL), 	 opNULL    ],
-  OBJECT_IDENTIFIER => [ asn_encode_tag(ASN_OBJECT_ID),  opOBJID   ],
-  REAL		    => [ asn_encode_tag(ASN_REAL), 	 opREAL    ],
-  ENUMERATED	    => [ asn_encode_tag(ASN_ENUMERATED), opINTEGER ],
-  ENUM		    => [ asn_encode_tag(ASN_ENUMERATED), opINTEGER ],
+  BOOLEAN	    => [ asn_encode_tag(ASN_BOOLEAN),		opBOOLEAN ],
+  INTEGER	    => [ asn_encode_tag(ASN_INTEGER),		opINTEGER ],
+  BIT_STRING	    => [ asn_encode_tag(ASN_BIT_STR),		opBITSTR  ],
+  OCTET_STRING	    => [ asn_encode_tag(ASN_OCTET_STR),		opSTRING  ],
+  STRING	    => [ asn_encode_tag(ASN_OCTET_STR),		opSTRING  ],
+  NULL 		    => [ asn_encode_tag(ASN_NULL),		opNULL    ],
+  OBJECT_IDENTIFIER => [ asn_encode_tag(ASN_OBJECT_ID),		opOBJID   ],
+  REAL		    => [ asn_encode_tag(ASN_REAL),		opREAL    ],
+  ENUMERATED	    => [ asn_encode_tag(ASN_ENUMERATED),	opINTEGER ],
+  ENUM		    => [ asn_encode_tag(ASN_ENUMERATED),	opINTEGER ],
+  'RELATIVE-OID'    => [ asn_encode_tag(ASN_RELATIVE_OID),	opROID	  ],
 
   SEQUENCE	    => [ asn_encode_tag(ASN_SEQUENCE | ASN_CONSTRUCTOR), opSEQUENCE ],
   SET               => [ asn_encode_tag(ASN_SET      | ASN_CONSTRUCTOR), opSET ],
@@ -68,7 +73,7 @@ sub explicit {
   \@seq;
 }
 
-# 66 "y.tab.pl"
+# 74 "y.tab.pl"
 
 sub constWORD () { 1 }
 sub constCLASS () { 2 }
@@ -191,7 +196,7 @@ sub constYYFINAL () { 4 }
 
 
 sub constYYMAXTOKEN () { 18 }
-# 254 "y.tab.pl"
+# 262 "y.tab.pl"
 
 sub yyclearin { $yychar = -1; }
 sub yyerrok { $yyerrflag = 0; }
@@ -230,7 +235,7 @@ sub yy_err_recover
   else
   {
     return (1) if $yychar == 0;
-# 305 "y.tab.pl"
+# 313 "y.tab.pl"
 
     $yychar = -1;
     next yyloop;
@@ -262,7 +267,7 @@ yyloop: while(1)
       if ($yychar < 0)
       {
         if (($yychar = &yylex) < 0) { $yychar = 0; }
-# 344 "y.tab.pl"
+# 352 "y.tab.pl"
 
       }
       if (($yyn = $yysindex[$yystate]) && ($yyn += $yychar) >= 0 &&
@@ -302,13 +307,13 @@ my $label = "State$yyn";
 goto $label if exists $yystate{$label};
 last switch;
 State1: {
-# 86 "parser.y"
+# 94 "parser.y"
 
 { $yyval = { '' => $yyvs[$yyvsp-0] }; 
 last switch;
 } }
 State3: {
-# 91 "parser.y"
+# 99 "parser.y"
 
 {
 		  $yyval = { $yyvs[$yyvsp-2], [$yyvs[$yyvsp-0]] };
@@ -316,7 +321,7 @@ State3: {
 last switch;
 } }
 State4: {
-# 95 "parser.y"
+# 103 "parser.y"
 
 {
 		  $yyval=$yyvs[$yyvsp-3];
@@ -325,7 +330,7 @@ State4: {
 last switch;
 } }
 State5: {
-# 102 "parser.y"
+# 110 "parser.y"
 
 {
 		  $yyvs[$yyvsp-1]->[cTAG] = $yyvs[$yyvsp-3];
@@ -334,7 +339,7 @@ State5: {
 last switch;
 } }
 State11: {
-# 116 "parser.y"
+# 124 "parser.y"
 
 {
 		  @{$yyval = []}[cTYPE,cCHILD] = ('COMPONENTS', $yyvs[$yyvsp-0]);
@@ -342,7 +347,7 @@ State11: {
 last switch;
 } }
 State14: {
-# 126 "parser.y"
+# 134 "parser.y"
 
 {
 		  $yyvs[$yyvsp-0]->[cTAG] = $yyvs[$yyvsp-2];
@@ -352,7 +357,7 @@ State14: {
 last switch;
 } }
 State18: {
-# 139 "parser.y"
+# 147 "parser.y"
 
 {
 		  @{$yyval = []}[cTYPE,cCHILD] = ('SEQUENCE', $yyvs[$yyvsp-1]);
@@ -360,7 +365,7 @@ State18: {
 last switch;
 } }
 State19: {
-# 143 "parser.y"
+# 151 "parser.y"
 
 {
 		  @{$yyval = []}[cTYPE,cCHILD] = ('SET', $yyvs[$yyvsp-1]);
@@ -368,7 +373,7 @@ State19: {
 last switch;
 } }
 State20: {
-# 147 "parser.y"
+# 155 "parser.y"
 
 {
 		  @{$yyval = []}[cTYPE,cCHILD] = ('CHOICE', $yyvs[$yyvsp-1]);
@@ -376,7 +381,7 @@ State20: {
 last switch;
 } }
 State21: {
-# 153 "parser.y"
+# 161 "parser.y"
 
 {
 		  @{$yyval = []}[cTYPE] = ('ENUM');
@@ -384,7 +389,7 @@ State21: {
 last switch;
 } }
 State27: {
-# 166 "parser.y"
+# 174 "parser.y"
 
 {
 		  @{$yyval = []}[cTYPE] = ($yyvs[$yyvsp-0]);
@@ -392,19 +397,19 @@ State27: {
 last switch;
 } }
 State28: {
-# 171 "parser.y"
+# 179 "parser.y"
 
 { $yyval = $yyvs[$yyvsp-0]; 
 last switch;
 } }
 State29: {
-# 172 "parser.y"
+# 180 "parser.y"
 
 { $yyval = $yyvs[$yyvsp-1]; 
 last switch;
 } }
 State30: {
-# 176 "parser.y"
+# 184 "parser.y"
 
 {
 		  $yyval = [ $yyvs[$yyvsp-0] ];
@@ -412,7 +417,7 @@ State30: {
 last switch;
 } }
 State31: {
-# 180 "parser.y"
+# 188 "parser.y"
 
 {
 		  push @{$yyval=$yyvs[$yyvsp-2]}, $yyvs[$yyvsp-0];
@@ -420,7 +425,7 @@ State31: {
 last switch;
 } }
 State32: {
-# 184 "parser.y"
+# 192 "parser.y"
 
 {
 		  push @{$yyval=$yyvs[$yyvsp-2]}, $yyvs[$yyvsp-0];
@@ -428,7 +433,7 @@ State32: {
 last switch;
 } }
 State33: {
-# 190 "parser.y"
+# 198 "parser.y"
 
 {
 		  @{$yyval=$yyvs[$yyvsp-0]}[cVAR,cTAG] = ($yyvs[$yyvsp-3],$yyvs[$yyvsp-2]);
@@ -437,19 +442,19 @@ State33: {
 last switch;
 } }
 State34: {
-# 197 "parser.y"
+# 205 "parser.y"
 
 { $yyval = $yyvs[$yyvsp-0]; 
 last switch;
 } }
 State35: {
-# 198 "parser.y"
+# 206 "parser.y"
 
 { $yyval = $yyvs[$yyvsp-1]; 
 last switch;
 } }
 State36: {
-# 202 "parser.y"
+# 210 "parser.y"
 
 {
 		  $yyval = [ $yyvs[$yyvsp-0] ];
@@ -457,7 +462,7 @@ State36: {
 last switch;
 } }
 State37: {
-# 206 "parser.y"
+# 214 "parser.y"
 
 {
 		  push @{$yyval=$yyvs[$yyvsp-2]}, $yyvs[$yyvsp-0];
@@ -465,7 +470,7 @@ State37: {
 last switch;
 } }
 State38: {
-# 210 "parser.y"
+# 218 "parser.y"
 
 {
 		  push @{$yyval=$yyvs[$yyvsp-2]}, $yyvs[$yyvsp-0];
@@ -473,7 +478,7 @@ State38: {
 last switch;
 } }
 State39: {
-# 216 "parser.y"
+# 224 "parser.y"
 
 {
 		  @{$yyval=$yyvs[$yyvsp-1]}[cOPT] = ($yyvs[$yyvsp-0]);
@@ -481,7 +486,7 @@ State39: {
 last switch;
 } }
 State43: {
-# 225 "parser.y"
+# 233 "parser.y"
 
 {
 		  @{$yyval=$yyvs[$yyvsp-0]}[cVAR,cTAG] = ($yyvs[$yyvsp-3],$yyvs[$yyvsp-2]);
@@ -491,7 +496,7 @@ State43: {
 last switch;
 } }
 State45: {
-# 232 "parser.y"
+# 240 "parser.y"
 
 {
 		  @{$yyval=$yyvs[$yyvsp-0]}[cTAG] = ($yyvs[$yyvsp-2]);
@@ -500,72 +505,72 @@ State45: {
 last switch;
 } }
 State46: {
-# 238 "parser.y"
+# 246 "parser.y"
 
 { $yyval = undef; 
 last switch;
 } }
 State47: {
-# 239 "parser.y"
+# 247 "parser.y"
 
 { $yyval = 1;     
 last switch;
 } }
 State48: {
-# 243 "parser.y"
+# 251 "parser.y"
 
 { $yyval = undef; 
 last switch;
 } }
 State50: {
-# 247 "parser.y"
+# 255 "parser.y"
 
 { $yyval = undef; 
 last switch;
 } }
 State51: {
-# 248 "parser.y"
+# 256 "parser.y"
 
 { $yyval = 1;     
 last switch;
 } }
 State52: {
-# 249 "parser.y"
+# 257 "parser.y"
 
 { $yyval = 0;     
 last switch;
 } }
 State53: {
-# 252 "parser.y"
-
-{
-last switch;
-} }
-State54: {
-# 253 "parser.y"
-
-{
-last switch;
-} }
-State55: {
-# 256 "parser.y"
-
-{
-last switch;
-} }
-State56: {
-# 259 "parser.y"
-
-{
-last switch;
-} }
-State57: {
 # 260 "parser.y"
 
 {
 last switch;
 } }
-# 607 "y.tab.pl"
+State54: {
+# 261 "parser.y"
+
+{
+last switch;
+} }
+State55: {
+# 264 "parser.y"
+
+{
+last switch;
+} }
+State56: {
+# 267 "parser.y"
+
+{
+last switch;
+} }
+State57: {
+# 268 "parser.y"
+
+{
+last switch;
+} }
+# 615 "y.tab.pl"
 
     } # switch
     $yyssp -= $yym;
@@ -584,7 +589,7 @@ last switch;
       if ($yychar < 0)
       {
         if (($yychar = &yylex) < 0) { $yychar = 0; }
-# 633 "y.tab.pl"
+# 641 "y.tab.pl"
 
       }
       return $yyvs[$yyvsp] if $yychar == 0;
@@ -605,7 +610,7 @@ last switch;
     $yyvs[++$yyvsp] = $yyval;
   } # yyloop
 } # yyparse
-# 264 "parser.y"
+# 272 "parser.y"
 
 
 my %reserved = (
@@ -826,6 +831,8 @@ sub yylex {
 	    (?:OCTET|BIT)\s+STRING
 	   |
 	    OBJECT\s+IDENTIFIER
+	   |
+	    RELATIVE-OID
 	  )\b
 	|
 	  (\w+)
@@ -890,14 +897,14 @@ sub yyerror {
 
 1;
 
-# 937 "y.tab.pl"
+# 947 "y.tab.pl"
 
-%yystate = ('State20','','State11','','State30','','State21','','State31',
-'','State50','','State32','','State14','','State51','','State33','',
-'State52','','State43','','State34','','State53','','State35','','State54',
-'','State45','','State36','','State18','','State27','','State55','',
-'State46','','State37','','State19','','State28','','State56','','State47',
-'','State38','','State29','','State57','','State48','','State39','',
-'State1','','State3','','State4','','State5','');
+%yystate = ('State20','','State21','','State43','','State27','','State28',
+'','State45','','State29','','State46','','State47','','State48','',
+'State1','','State3','','State4','','State5','','State11','','State14','',
+'State30','','State31','','State32','','State33','','State18','','State34',
+'','State50','','State19','','State35','','State51','','State36','',
+'State52','','State37','','State53','','State38','','State54','','State39',
+'','State55','','State56','','State57','');
 
 1;
